#pragma once

#include "stdafx.h"
#include "iis4ruby_cmn_string.h"
#include "Message.h"

//
// Export Function
//

void ReportEventLogForSystem(
					const char* pFile, const int iLine, const char* pDate, const char* pTime ) ;
void ReportEventLog( WORD wType, WORD wCategory, DWORD dwEventID,
					const char* pFile, const int iLine, const char* pDate, const char* pTime, LPCTSTR pMsg ) ;
void InitCmnLib( HMODULE hModule ) ;

//
// Log Macro
//

#ifdef _DEBUG

#define REL_SUCC( _event_category_, _err_code_, _msg_ )	\
	ReportEventLog( EVENTLOG_SUCCESS, _event_category_, _err_code_, __FILE__, __LINE__, __DATE__, __TIME__, _msg_  )
#define REL_INFO( _event_category_, _err_code_, _msg_ )	\
	ReportEventLog( EVENTLOG_INFORMATION_TYPE, _event_category_, _err_code_, __FILE__, __LINE__, __DATE__, __TIME__, _msg_  )
#define REL_WARN( _event_category_, _err_code_, _msg_ )	\
	ReportEventLog( EVENTLOG_WARNING_TYPE, _event_category_, _err_code_, __FILE__, __LINE__, __DATE__, __TIME__, _msg_  )
#define REL_ERR( _event_category_, _err_code_, _msg_ )	\
	ReportEventLog( EVENTLOG_ERROR_TYPE, _event_category_, _err_code_, __FILE__, __LINE__, __DATE__, __TIME__, _msg_  )

#else

#define REL_SUCC( _event_category_, _err_code_, _msg_ )
#define REL_INFO( _event_category_, _err_code_, _msg_ )
#define REL_WARN( _event_category_, _err_code_, _msg_ )	\
	ReportEventLog( EVENTLOG_WARNING_TYPE, _event_category_, _err_code_, __FILE__, __LINE__, __DATE__, __TIME__, _msg_  )
#define REL_ERR( _event_category_, _err_code_, _msg_ )	\
	ReportEventLog( EVENTLOG_ERROR_TYPE, _event_category_, _err_code_, __FILE__, __LINE__, __DATE__, __TIME__, _msg_  )

#endif

#define REL_SYSTEM()	\
	ReportEventLogForSystem( __FILE__, __LINE__, __DATE__, __TIME__ )
#define CHK_RET_REL_SYSTEM( _bool_ )	\
	if( ! _bool_ ) {	\
		REL_SYSTEM() ;	\
		return FALSE ; \
	}

#define BEGIN_TRY( _flg_ )	\
	BOOL _flg_	= TRUE ;	\
	try {
#define END_TRY( _flg_, _no_throw_ )	\
	}	\
	catch( std::exception& ex ) {	\
		_flg_	= FALSE ;	\
		REL_ERR( CAT_ETC, MSG_SYS, A2T( ex.what() ) ) ;	\
		if( !_no_throw_ ) throw ex ;	\
	}	\
	catch( ... ) {	\
		_flg_	= FALSE ;	\
		REL_SYSTEM() ;	\
		if( !_no_throw_ ) throw ;	\
	}

//
// Event Log
//

#define MAX_LEN_EVENT_TEXT	1024
#define CNT_EVENT_CATEGORY	5

#define EVENT_DATASOURCE _T("IIS4RUBY")

const TCHAR EVENTLOG_STRINGS[][ MAX_LEN_EVENT_TEXT ]	= {
	{ _T("Source(%s),Line(%d),Date(%s),Time(%s)") },
	{ _T("%s") },
} ;

#define REG_EVENTLOG_ROOT _T("SYSTEM\\CurrentControlSet\\Services\\EventLog\\Application\\")
#define REG_EVENTLOG_KEY_EVT_MSG_FILE _T("EventMessageFile")
#define REG_EVENTLOG_KEY_CAT_MSG_FILE _T("CategoryMessageFile")
#define REG_EVENTLOG_KEY_PARAM_MSG_FILE _T("ParameterMessageFile")
#define REG_EVENTLOG_KEY_TYPES_SUPPORTED _T("TypesSupported")
#define REG_EVENTLOG_KEY_CAT_COUNT _T("CategoryCount")

